"use strict";
///<reference path="./d/externals.d.ts" />
var ipRegex = require('ip-regex');
var extend = require('extend');
//built-in transforms
//url transform
exports.url = {
    pattern: function (options) {
        return urlRegExp(options);
    },
    transform: function (options, text, scheme) {
        var url = text;
        if (scheme === "") {
            url = "http://" + url;
        }
        return extend(options.url.attributes, {
            href: url
        });
    },
};
//generate url regexp
function urlRegExp(options) {
    //utility
    /// non-space printables (does NOT include ASCII chars)
    var nonspaces = "[\\u00a1-\\u167f\\u1681-\\u180d\\u180f-\\u1fff\\u200c-\\u202e\\u2030-\\u205e\\u2060-\\u2fff\\u3001-\\ufefe\\uff00-\\uffff]";
    var url = options.url;
    //schemeのRegExp
    var scheme, schemes = url.schemes;
    if (!url.requireSchemes) {
        scheme = "";
    }
    else if (Array.isArray(schemes)) {
        //some protocolsがallowされている
        scheme = "(?:" + schemes.join("|") + ")://";
    }
    else if (url.schemes === "*") {
        //any
        scheme = "[a-z]+://";
    }
    else {
        throw new Error("Unrecognized value for options.url.schemes");
    }
    scheme = "(" + scheme + ")";
    //auth RegExp
    var auth = "(?:\\S+(?::\\S*)?@)?";
    //hostname
    var hostpart = "(?:(?:(?:[0-9a-z]|" + nonspaces + ")-*)*(?:[0-9a-z]|" + nonspaces + ")+)";
    var hostname = "(?:" + hostpart + "(?:\\." + hostpart + ")+\\.?)";
    var host = "(?:localhost|" + ipRegex.v4().source + "|\\[" + ipRegex.v6().source + "\\]|" + hostname + ")";
    //port
    var port = "(?:(?::\\d{1,5})?)";
    //path
    var path = "(?:(?:/\\S*)?)";
    return new RegExp(scheme + auth + host + port + path, "ig");
}
